# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ReasoningContent"]


class ReasoningContent(BaseModel):
    """Sent via the Anthropic Messages API"""

    is_native: bool
    """
    Whether the reasoning content was generated by a reasoner model that processed
    this step.
    """

    reasoning: str
    """The intermediate reasoning or thought process content."""

    signature: Optional[str] = None
    """A unique identifier for this reasoning step."""

    type: Optional[Literal["reasoning"]] = None
    """Indicates this is a reasoning/intermediate step."""
