# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .conversation import Conversation

__all__ = ["ConversationListResponse"]

ConversationListResponse: TypeAlias = List[Conversation]
