# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List
from typing_extensions import TypeAlias

__all__ = ["TraceRetrieveResponse"]

TraceRetrieveResponse: TypeAlias = List[Dict[str, object]]
